@@ -20,6 +20,10 @@
 # ==============================================================================
 #
 # SPDX-License-Identifier: Apache-2.0
+#
+# Versioning
+__version__ = "0.20.0"
+
 # Imports (wildcard)
 from .data import *
 from .distributed import *
@@ -38,6 +42,10 @@ del fashionmnist_dataloader_dispatcher
 del imagenette_dataloader_dispatcher
 del mnist_dataloader_dispatcher
 del kmnist_dataloader_dispatcher
+del pathmnist_dataloader_dispatcher
+del octmnist_dataloader_dispatcher
+del tissuemnist_dataloader_dispatcher
+del tinyimagenet_dataloader_dispatcher
 
 # Deletions (from .distributed)
 del reduce_accumulate_keepalive
@@ -47,6 +55,8 @@ del slurm_nccl_env
 del ArgMaxLayer
 del BasicAE
 del BasicVAE
+del beta_gaussian_kldiv
+del var_of_lap
 del BinarizeLayer
 del BrokenReLU
 del CausalConv1d
@@ -57,14 +67,16 @@ del CoordConv2d
 del CoordConv3d
 del DeepRBL
 del FCBlock
-del FCBlockLegacy
 del FieldTransform
 del FlatChannelize2DLayer
 del GaussianReparameterizerSampler
+del GaussianReparameterizerSamplerLegacy
 del InnerProduct
 del KWTA1d
 del KWTA2d
 del Mish
+del multilasso
+del multiridge
 del MultiSolvePoissonTensor
 del NNEnsemble
 del PoissonNetCifar
@@ -81,14 +93,31 @@ del SirenSine
 del SmeLU
 del SolvePoisson
 del SolvePoissonTensor
+del SwiGLU
+del TupleDecouple
+del SilhouetteScore
+del Concatenate
+del DuplexLinearNeck
+del SharedDuplexLinearNeck
 del WideResNet
 del beta_reco_bce
+del beta_reco_bce_splitout
+del lexsemble
 del build_repeated_sequential
 del field_transform
 del mishlayer_init
 del pixelwise_bce_mean
 del pixelwise_bce_sum
 del oldtranspose
+del silhouette_score
+del cummatmul
+del tensor_replicate
+del logit_to_prob
+del bisided_thresholding
+del convstem_block
+del convnext_stem
+del smallconv_featurizer
+del MetaAILayerNorm
 
 # Deletions (from .optim)
 del AdaBound
@@ -107,6 +136,8 @@ del epochwise_onecycle
 del onecycle_lincos
 del onecycle_linlin
 del onecycle_linlin_updown
+del warmed_up_linneal
+del make_beta_scheduler
 del ralah_optim
 del tricyc1c
 del wfneal
