@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# ------------------------------------------------------------------------------
+# ──────────────────────────────────────────────────────────────────────────────
 #
 # Copyright (c) 2020-* Emanuele Ballarin <emanuele@ballarin.cc>
 #                      All Rights Reserved.
@@ -17,7 +17,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# ------------------------------------------------------------------------------
+# ──────────────────────────────────────────────────────────────────────────────
 # SPDX-License-Identifier: Apache-2.0
 from copy import deepcopy
 from typing import Tuple
@@ -27,14 +27,14 @@ from torch import nn as thnn
 
 import ebtorch.nn as ebthnn
 
-# ------------------------------------------------------------------------------
+# ──────────────────────────────────────────────────────────────────────────────
 
 __all__ = [
     "data_prep_dispatcher_1ch",
     "data_prep_dispatcher_3ch",
 ]
 
-# ------------------------------------------------------------------------------
+# ──────────────────────────────────────────────────────────────────────────────
 
 
 def data_prep_dispatcher_1ch(
@@ -49,6 +49,12 @@ def data_prep_dispatcher_1ch(
     elif dataset == "kmnist":
         mean: float = 0.1918
         std: float = 0.3483
+    elif dataset == "tissuemnist":
+        mean: float = 0.102
+        std: float = 0.08
+    elif dataset == "octmnist":
+        mean: float = 0.1889
+        std: float = 0.1694
     else:
         raise ValueError("Invalid dataset.")
     if post_flatten:
@@ -79,6 +85,15 @@ def data_prep_dispatcher_3ch(
     elif dataset == "imagenet":
         means: Tuple[float, float, float] = (0.485, 0.456, 0.406)
         stds: Tuple[float, float, float] = (0.229, 0.224, 0.225)
+    elif dataset == "pathmnist":
+        means: Tuple[float, float, float] = (0.7405, 0.533, 0.7058)
+        stds: Tuple[float, float, float] = (0.0723, 0.1038, 0.0731)
+    elif dataset == "svhn":
+        means: Tuple[float, float, float] = (0.5, 0.5, 0.5)
+        stds: Tuple[float, float, float] = (0.5, 0.5, 0.5)
+    elif dataset == "tinyimagenet":
+        means: Tuple[float, float, float] = (0.4802, 0.4481, 0.3975)
+        stds: Tuple[float, float, float] = (0.2302, 0.2265, 0.2262)
     else:
         raise ValueError("Invalid dataset.")
     if post_flatten:
