@@ -22,6 +22,7 @@
 # IMPORTS
 import copy
 from collections.abc import Callable
+from math import copysign
 from typing import List
 from typing import Optional
 from typing import Tuple
@@ -33,10 +34,14 @@ from torch import Tensor
 from torch.nn import functional as F
 from torch.utils.hooks import RemovableHandle
 
+from .functional import silhouette_score
+from .penalties import beta_gaussian_kldiv
+
 __all__ = [
     "pixelwise_bce_sum",
     "pixelwise_bce_mean",
     "beta_reco_bce",
+    "beta_reco_bce_splitout",
     "FCBlock",
     "CausalConv1d",
     "build_repeated_sequential",
@@ -52,12 +57,29 @@ __all__ = [
     "BasicAE",
     "BasicVAE",
     "Clamp",
+    "SwiGLU",
+    "TupleDecouple",
+    "SilhouetteScore",
+    "Concatenate",
+    "DuplexLinearNeck",
+    "SharedDuplexLinearNeck",
+    "GaussianReparameterizerSamplerLegacy",
+    "lexsemble",
 ]
 
 # CUSTOM TYPES
 realnum = Union[float, int]
 
 
+# Ensembling functions
+@torch.jit.script
+def lexsemble(x: Tensor, cls_dim: int = -2, ens_dim: int = -1) -> Tensor:
+    out: Tensor = torch.softmax(
+        torch.exp(x).sum(dim=ens_dim), dim=cls_dim - int(copysign(1, cls_dim))
+    )
+    return torch.log(out / (1 - out))
+
+
 # Loss functions
 @torch.jit.script
 def pixelwise_bce_sum(lhs: Tensor, rhs: Tensor) -> Tensor:
@@ -75,13 +97,26 @@ def beta_reco_bce(
     input_orig: Tensor,
     mu: Tensor,
     sigma: Tensor,
-    beta: float = 0.5,
+    beta: float = 1.0,
 ):
-    kldiv = (beta * (torch.pow(mu, 2) + torch.exp(sigma) - sigma - 1)).sum()
+    kldiv = beta_gaussian_kldiv(mu, sigma, beta)
     pwbce = pixelwise_bce_sum(input_reco, input_orig)
     return pwbce + kldiv
 
 
+@torch.jit.script
+def beta_reco_bce_splitout(
+    input_reco: Tensor,
+    input_orig: Tensor,
+    mu: Tensor,
+    sigma: Tensor,
+    beta: float = 1.0,
+):
+    kldiv = beta_gaussian_kldiv(mu, sigma, beta)
+    pwbce = pixelwise_bce_sum(input_reco, input_orig)
+    return pwbce + kldiv, pwbce, kldiv
+
+
 # Utility functions
 def _rbm_mask_generator(
     size: int, ood_width: int = 0, dens: float = 1.0, rand_diag: bool = False
@@ -165,7 +200,9 @@ class FCBlock(nn.Module):
         error_uneven_size: str = (
             "The length of lists of arguments must be the same across them."
         )
-        error_illegal_dropout: str = "The 'dropout' argument must be either False, a float, or an iterable of floats and/or False."
+        error_illegal_dropout: str = (
+            "The 'dropout' argument must be either False, a float, or an iterable of floats and/or False."
+        )
 
         # Default cases
         if bias is None:
@@ -360,7 +397,6 @@ class CausalConv1d(nn.Conv1d):
 # Reparameterizer / Sampler for (C)VAEs & co.
 
 
-# Do not make static, regardless of what the linter/analyzer says... ;)
 def _gauss_reparameterize_sample(
     z_mu: Tensor, z_log_var: Tensor, device: Optional[torch.DeviceObjType] = None
 ) -> Tensor:
@@ -375,17 +411,27 @@ def _gauss_reparameterize_sample(
     ).to(device)
 
 
-class GaussianReparameterizerSampler(nn.Module):
+class GaussianReparameterizerSamplerLegacy(nn.Module):
     def __init__(self):  # skipcq: PYL-W0235
         super().__init__()
 
-    # Do not make static!
+    # noinspection PyMethodMayBeStatic
     def forward(self, z_mu: Tensor, z_log_var: Tensor) -> Tensor:  # skipcq: PYL-R0201
         return _gauss_reparameterize_sample(z_mu, z_log_var)
 
 
-class SGRUHCell(nn.Module):
+class GaussianReparameterizerSampler(nn.Module):
+    def __init__(self):
+        super().__init__()
 
+    # noinspection PyMethodMayBeStatic
+    def forward(self, z_mu: torch.Tensor, z_log_var: torch.Tensor) -> torch.Tensor:
+        return z_mu + torch.randn_like(z_mu, device=z_mu.device) * torch.exp(
+            z_log_var * 0.5
+        )
+
+
+class SGRUHCell(nn.Module):
     """
     Stateful (i.e. implicit hidden state) GRU HyperCell (i.e. arbitrary
     time-order and depth) with ReadIn and ReadOut custom heads
@@ -490,7 +536,7 @@ class ArgMaxLayer(nn.Module):
     def __init__(self) -> None:
         super().__init__()
 
-    # Do not make static!
+    # noinspection PyMethodMayBeStatic
     def forward(self, x: Tensor) -> Tensor:  # skipcq: PYL-R0201
         return torch.argmax(x, dim=1)
 
@@ -528,7 +574,7 @@ class Clamp(nn.Module):
         return x.clamp(self.min, self.max)
 
 
-# ------------------------------------------------------------------------------
+# ──────────────────────────────────────────────────────────────────────────────
 class RBLinear(nn.Linear):
     """
     Random Band Linear Layer
@@ -806,9 +852,7 @@ class BasicVAE(nn.Module):
         self.extract_z: bool = extract_z
         self.extract_mv: bool = extract_mv
 
-    def forward(
-        self, x: Tensor
-    ) -> Union[
+    def forward(self, x: Tensor) -> Union[
         Tensor,
         Tuple[Tensor, Tensor],
         Tuple[Tensor, Tensor, Tensor],
@@ -828,3 +872,85 @@ class BasicVAE(nn.Module):
             return y, mean, logvar
         else:
             return y
+
+
+class SwiGLU(nn.Module):
+    def __init__(
+        self,
+        dim: int,
+        hidden_dim: int,
+        bias: bool = False,
+    ):
+        super().__init__()
+
+        self.w1: nn.Linear = nn.Linear(dim, hidden_dim, bias=bias)
+        self.w2: nn.Linear = nn.Linear(hidden_dim, dim, bias=bias)
+        self.w3: nn.Linear = nn.Linear(dim, hidden_dim, bias=bias)
+
+    def forward(self, x):
+        return self.w2(F.silu(self.w1(x)) * self.w3(x))
+
+
+class TupleDecouple(nn.Module):
+    def __init__(self, module: nn.Module, idx: int = 0) -> None:
+        super().__init__()
+        self.module: nn.Module = module
+        self.idx: int = idx
+
+    def forward(self, xtuple: Tuple[Tensor, ...]) -> Tuple[Tensor, ...]:
+        return (
+            *xtuple[: self.idx],
+            self.module(xtuple[self.idx]),
+            *xtuple[self.idx + 1 :],
+        )
+
+
+class SilhouetteScore(nn.Module):
+    """
+    Layerized computation of the Silhouette Score.
+    """
+
+    @staticmethod
+    def forward(features: Tensor, labels: Tensor) -> Tensor:
+        return silhouette_score(features, labels)
+
+
+class Concatenate(nn.Module):
+    def __init__(self, dim: int = 1, flatten: bool = False):
+        super().__init__()
+        self.dim: int = dim
+        self.flatten: bool = flatten
+
+    def forward(self, tensors: Union[Tuple[torch.Tensor, ...], List[torch.Tensor]]):
+        tensors = (
+            [tensor.flatten(start_dim=1) for tensor in tensors]
+            if self.flatten
+            else tensors
+        )
+        return torch.cat(tensors, dim=self.dim)
+
+
+class DuplexLinearNeck(nn.Module):
+    def __init__(self, in_dim: int, latent_dim: int):
+        super().__init__()
+        self.x_to_mu: nn.Linear = nn.Linear(in_dim, latent_dim)
+        self.x_to_log_var: nn.Linear = nn.Linear(in_dim, latent_dim)
+
+    def forward(
+        self, xc: Union[Tuple[torch.Tensor, ...], List[torch.Tensor]]
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
+        cxc: torch.Tensor = torch.cat(xc, dim=1)
+        return self.x_to_mu(cxc), self.x_to_log_var(cxc)
+
+
+class SharedDuplexLinearNeck(nn.Module):
+    def __init__(self, in_dim: int, latent_dim: int):
+        super().__init__()
+        self.shared_layer: nn.Linear = nn.Linear(in_dim, 2 * latent_dim)
+
+    def forward(
+        self, xc: Union[Tuple[torch.Tensor, ...], List[torch.Tensor]]
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
+        cxc: torch.Tensor = torch.cat(xc, dim=1)
+        # noinspection PyTypeChecker
+        return torch.chunk(self.shared_layer(cxc), 2, dim=1)
