@@ -25,11 +25,13 @@ from .adabound import AdaBound
 from .adahessian import AdaHessian
 from .adan import Adan
 from .custom import epochwise_onecycle
+from .custom import make_beta_scheduler
 from .custom import onecycle_lincos
 from .custom import onecycle_linlin
 from .custom import onecycle_linlin_updown
 from .custom import ralah_optim
 from .custom import tricyc1c
+from .custom import warmed_up_linneal
 from .custom import wfneal
 from .lamb import Lamb
 from .lion import Lion
